import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;
import static ru.ifmo.testlib.Outcome.Type.*;

public class Check implements Checker {
    public Outcome outcome(Outcome.Type type, final String format, final Object... args) {
        return new Outcome(type, String.format(format, args));
    }

    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int testsCount = inf.nextInt();
        for (int test = 0; test < testsCount; test++) {
            String pref = "Test #" + (test + 1) + ": ";
            int n = inf.nextInt();
            int m = inf.nextInt();
            String pAns = ouf.nextToken();
            String jAns = ans.nextToken();
            int[] vcolors = new int[n];
            for (int i = 0; i < n; i++) {
                vcolors[i] = inf.nextInt();
            }
            int[] vs = new int[m];
            int[] us = new int[m];
            int[] cs = new int[m];
            for (int i = 0; i < m; i++) {
                vs[i] = inf.nextInt() - 1;
                us[i] = inf.nextInt() - 1;
                cs[i] = inf.nextInt();
            }

            int[] p = new int[n];
            int[] j = new int[n];
            if (pAns.equals("YES")) {
                for (int i = 0; i < n; i++) {
                    p[i] = ouf.nextInt();
                }
            }
            if (jAns.equals("YES")) {
                for (int i = 0; i < n; i++) {
                    j[i] = ans.nextInt();
                }
            }

            boolean jCheck = check(jAns, j, vs, us, cs, vcolors);
            boolean pCheck = check(pAns, p, vs, us, cs, vcolors);
            if (!jCheck) {
                return outcome(FAIL, pref + "Jury has wrong answer");
            }
            if (pAns.equals("YES") && pCheck && jAns.equals("NO")) {
                return outcome(FAIL, pref + "Participant found answer, but jury doesn't");
            }
            if (!pAns.equals(jAns)) {
                return outcome(WA, String.format("%d-th answer is wrong - expected %s, found %s, [n = %d, m = %d]", test + 1, jAns, pAns, n, m));
            }
            if (pAns.equals("YES") && !pCheck) {
                return outcome(WA, String.format("%d-th answer is correct - YES, but contract is wrong", test + 1));
            }
        }
        return outcome(OK, String.format("Correct answer, %d tests", testsCount));
    }

    boolean check(String answer, int[] ans, int[] vs, int[] us, int[] cs, int[] vcolors) {
        if (answer.equals("NO")) {
            return true;
        }
        int cnt = 0;
        for (int i = 0; i < ans.length; i++) {
            if (ans[i] == 1) {
                cnt++;
            }
        }
        if (2 * cnt != ans.length) {
            return false;
        }
        for (int i = 0; i < vs.length; i++) {
            if (cs[i] == 0 && ans[vs[i]] != ans[us[i]]) {   
                return false;
            }
            if (cs[i] == 1 && ans[vs[i]] == ans[us[i]]) {   
                return false;
            }
        }
        for (int i = 0; i < ans.length; i++) {
            if (vcolors[i] != 0 && ans[i] != vcolors[i]) {
                return false;
            }
        }
        return true;
    }
}
