import java.io.*;
import java.util.*;

public class teams_av {

	FastScanner in;
	PrintWriter out;

	class Edge {
		int from, to, len;

		public Edge(int from, int to, int len) {
			super();
			this.from = from;
			this.to = to;
			this.len = len;
		}
	}

	List<Edge>[] graph;
	int[] color;
	boolean bicolored;

	void dfs(int u, int c, List<Integer> left, List<Integer> right) {
		color[u] = c;
		if (c == 0) {
			left.add(u);
		} else {
			right.add(u);
		}

		for (Edge e : graph[u]) {
			if (color[e.to] == -1) {
				dfs(e.to, (c + e.len) % 2, left, right);
			} else {
				if (color[e.to] != (c + e.len) % 2) {
					bicolored = false;
				}
			}
		}
	}

	class Component {
		List<Integer> first, second;
		boolean canFirst, canSecond;

		public Component(List<Integer> first, List<Integer> second) {
			this.first = first;
			this.second = second;
		}

	}

	void solve() {
		int n = in.nextInt(), m = in.nextInt();

		int[] team = new int[n];
		for (int i = 0; i < n; i++) {
			team[i] = in.nextInt() - 1;
		}
		graph = new List[n];
		for (int i = 0; i < n; i++) {
			graph[i] = new ArrayList<>();
		}

		for (int i = 0; i < m; i++) {
			int from = in.nextInt() - 1, to = in.nextInt() - 1, len = in
					.nextInt();
			graph[from].add(new Edge(from, to, len));
			graph[to].add(new Edge(to, from, len));
		}

		color = new int[n];
		Arrays.fill(color, -1);
		bicolored = true;

		List<Component> comps = new ArrayList<>();
		for (int i = 0; i < n; i++) {
			if (color[i] == -1) {
				List<Integer> left = new ArrayList<>();
				List<Integer> right = new ArrayList<>();
				dfs(i, 0, left, right);
				if (!bicolored) {
					out.println("NO");
					return;
				}

				Component comp = new Component(left, right);
				{
					comp.canFirst = true;
					for (int p : comp.first) {
						if (team[p] == 1) {
							comp.canFirst = false;
						}
					}
					for (int p : comp.second) {
						if (team[p] == 0) {
							comp.canFirst = false;
						}
					}
				}
				{
					comp.canSecond = true;
					for (int p : comp.first) {
						if (team[p] == 0) {
							comp.canSecond = false;
						}
					}
					for (int p : comp.second) {
						if (team[p] == 1) {
							comp.canSecond = false;
						}
					}
				}
				comps.add(comp);
			}
		}

		boolean[][] dp = new boolean[comps.size() + 1][n + 1];
		boolean[][] prev = new boolean[comps.size() + 1][n + 1];
		dp[0][0] = true;
		for (int i = 0; i < comps.size(); i++) {
			Component comp = comps.get(i);
			for (int j = 0; j <= n; j++) {
				if (j >= comp.first.size() && comp.canFirst && dp[i][j - comp.first.size()]) {
					dp[i + 1][j] = true;
					prev[i + 1][j] = false;
				}
				if (j >= comp.second.size() && comp.canSecond && dp[i][j - comp.second.size()]) {
					dp[i + 1][j] = true;
					prev[i + 1][j] = true;
				}
			}
		}

		if (n % 2 == 1 || !dp[comps.size()][n / 2]) {
			out.println("NO");
		} else {
			out.println("YES");

			List<Integer> firstTeam = new ArrayList<>(), secondTeam = new ArrayList<>();

			int cur = n / 2;
			for (int i = comps.size() - 1; i >= 0; i--) {
				Component comp = comps.get(i);
				if (!prev[i + 1][cur]) {
					firstTeam.addAll(comp.first);
					secondTeam.addAll(comp.second);
					cur -= comp.first.size();
				} else {
					firstTeam.addAll(comp.second);
					secondTeam.addAll(comp.first);
					cur -= comp.second.size();
				}
			}

			int[] ans = new int[n];
			for (int p : firstTeam) {
				ans[p] = 1;
			}
			for (int p : secondTeam) {
				ans[p] = 2;
			}
			for (int i : ans) {
				out.print(i + " ");
			}
			out.println();
		}
	}

	void run() {
		in = new FastScanner();
		out = new PrintWriter(System.out);
		int T = in.nextInt();
		for (int test = 0; test < T; test++) {
			solve();
		}
		out.close();
	}

	class FastScanner {
		BufferedReader br;
		StringTokenizer st;

		public FastScanner() {
			br = new BufferedReader(new InputStreamReader(System.in));
		}

		public FastScanner(String s) {
			try {
				br = new BufferedReader(new FileReader(s));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}

		String nextToken() {
			while (st == null || !st.hasMoreElements()) {
				try {
					st = new StringTokenizer(br.readLine());
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			return st.nextToken();
		}

		int nextInt() {
			return Integer.parseInt(nextToken());
		}

		long nextLong() {
			return Long.parseLong(nextToken());
		}

		double nextDouble() {
			return Double.parseDouble(nextToken());
		}
	}

	public static void main(String[] args) {
		new teams_av().run();
	}
}
