import java.io.*;
import java.util.*;

public class alley_bm2 {
    FastScanner in;
    PrintWriter out;

    class Segment implements Comparable<Segment> {
        int len;
        int curCnt;
        int curAnswer;

        Segment(int len) {
            this.len = len;
            curCnt = 0;
            updateAnswer();
        }

        void updateAnswer() {
            curAnswer = 1 + (len - 1) / (1 + curCnt);
        }

        @Override
        public int compareTo(Segment o) {
            return -Integer.compare(curAnswer, o.curAnswer);
        }
    }

    class Q implements Comparable<Q> {
        int id;
        int k;
        int ans;

        public Q(int id, int k) {
            super();
            this.id = id;
            this.k = k;
        }

        @Override
        public int compareTo(Q o) {
            return Integer.compare(k, o.k);
        }

    }

    void solve(int[] lengths, Q[] queries) {
        // long START = System.currentTimeMillis();
        Arrays.sort(queries);
        final int MAXK = (int) 1e6;
        PriorityQueue<Segment> segments = new PriorityQueue<>();
        for (int len : lengths) {
            segments.add(new Segment(len));
        }
        int lastAnswer = Integer.MAX_VALUE;
        int qIt = 0;
        for (int k = 1; k <= MAXK; k++) {
            Segment curSegment = segments.poll();
            while (qIt < queries.length && queries[qIt].k < k) {
                queries[qIt].ans = curSegment.curAnswer;
                qIt++;
            }
            lastAnswer = curSegment.curAnswer;
            if (curSegment.curCnt == curSegment.len - 1) {
                break;
            }
            curSegment.curCnt++;
            curSegment.updateAnswer();
            segments.add(curSegment);
        }
        // System.err.println("!!" + (System.currentTimeMillis() - START));
        // System.err.println("last Answer = " + lastAnswer);
        Arrays.sort(lengths);
        int[] cost = new int[lastAnswer + 1];
        for (int i = 0; i < lengths.length;) {
            int j = i;
            while (j != lengths.length && lengths[j] - lengths[i] == 0) {
                j++;
            }
            int len = lengths[i];
            int cnt = j - i;
            for (int myAns = 1; myAns <= lastAnswer; myAns++) {
                cost[myAns] += cnt * ((len - 1) / (myAns));
            }
            i = j;
        }

        // System.err.println("!!" + (System.currentTimeMillis() - START));
        for (; qIt < queries.length; qIt++) {
            int left = 0, right = lastAnswer;
            while (right - left > 1) {
                int mid = (left + right) / 2;
                if (cost[mid] <= queries[qIt].k) {
                    right = mid;
                } else {
                    left = mid;
                }
            }
            queries[qIt].ans = right;
        }

        // System.err.println("!!" + (System.currentTimeMillis() - START));
    }

    void solve() {
        // long START = System.currentTimeMillis();
        int n = in.nextInt();
        int m = in.nextInt();
        in.nextInt();
        int[] lengths = new int[n - 1];
        int last = in.nextInt();
        for (int i = 0; i < n - 1; i++) {
            int cur = in.nextInt();
            lengths[i] = (cur - last);
            last = cur;
        }
        Q[] q = new Q[m];
        for (int i = 0; i < m; i++) {
            q[i] = new Q(i, in.nextInt());
        }
        // System.err.println(System.currentTimeMillis() - START);
        solve(lengths, q);
        // System.err.println(System.currentTimeMillis() - START);
        int[] answer = new int[m];
        for (Q qq : q) {
            answer[qq.id] = qq.ans;
        }
        // System.err.println(System.currentTimeMillis() - START);
        for (int i = 0; i < m; i++) {
            out.println(answer[i]);
        }
        // System.err.println(System.currentTimeMillis() - START);
    }

    void run() {
        try {
            in = new FastScanner(new File("tests/02"));
            out = new PrintWriter(new File("E.out"));

            solve();

            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    void runIO() {

        in = new FastScanner(System.in);
        out = new PrintWriter(System.out);

        solve();

        out.close();
    }

    class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        public FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        public FastScanner(InputStream f) {
            br = new BufferedReader(new InputStreamReader(f));
        }

        String next() {
            while (st == null || !st.hasMoreTokens()) {
                String s = null;
                try {
                    s = br.readLine();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                if (s == null)
                    return null;
                st = new StringTokenizer(s);
            }
            return st.nextToken();
        }

        boolean hasMoreTokens() {
            while (st == null || !st.hasMoreTokens()) {
                String s = null;
                try {
                    s = br.readLine();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                if (s == null)
                    return false;
                st = new StringTokenizer(s);
            }
            return true;
        }

        int nextInt() {
            return Integer.parseInt(next());
        }

        long nextLong() {
            return Long.parseLong(next());
        }

        double nextDouble() {
            return Double.parseDouble(next());
        }
    }

    public static void main(String[] args) {
        new alley_bm2().runIO();
    }
}