import java.io.*;
import java.util.*;

public class stage_bm {
	FastScanner in;
	PrintWriter out;

	// 1:00
	// 1:19 -- debug
	// 1:32 -- finished
	
	int Z;

	class State {
		int k;
		int[] max;

		public State(int k) {
			super();
			this.k = k;
			this.max = new int[1 << k];
		}

		State addElement(int mask, int value) {
			State res = new State(k);
			for (int i = 0; i < 1 << k; i++) {
				res.max[i] = Math.max(res.max[i], max[i]);
				if ((i & mask) == 0) {
					res.max[i ^ mask] = Math.max(res.max[i ^ mask], res.max[i]
							+ value);
					if (res.max[i ^ mask] > Z) {
						res.max[i ^ mask] = Z;
					}
				}
			}
			return res;
		}

		int getMax() {
			return max[max.length - 1];
		}

		@Override
		public String toString() {
			return "State [max=" + Arrays.toString(max) + "]";
		}

	}

	class FIFO {
		ArrayList<State> first = new ArrayList<>();
		ArrayList<Integer> firstMask = new ArrayList<>();
		ArrayList<Integer> firstValue = new ArrayList<>();
		ArrayList<State> second = new ArrayList<>();

		int k;

		FIFO(int k) {
			this.k = k;
		}

		void add(int mask, int value) {
			State lastState = first.size() > 0 ? first.get(first.size() - 1)
					: new State(k);
			State newState = lastState.addElement(mask, value);
			first.add(newState);
			firstMask.add(mask);
			firstValue.add(value);
		}

		void remove() {
			if (second.size() == 0) {
				int k = first.get(first.size() - 1).k;
				State last = new State(k);
				while (first.size() > 0) {
					first.remove(first.size() - 1);
					int lastMask = firstMask.get(firstMask.size() - 1);
					int lastValue = firstValue.get(firstValue.size() - 1);
					firstMask.remove(firstMask.size() - 1);
					firstValue.remove(firstValue.size() - 1);
					last = last.addElement(lastMask, lastValue);
					second.add(last);
				}
			}
			second.remove(second.size() - 1);
		}

		int getMax() {
			if (first.size() == 0 && second.size() == 0) {
				return 0;
			}
			if (first.size() == 0) {
				return second.get(second.size() - 1).getMax();
			}
			if (second.size() == 0) {
				return first.get(first.size() - 1).getMax();
			}
			State s1 = first.get(first.size() - 1);
			State s2 = second.get(second.size() - 1);
			int max = 0;
			for (int i = 0; i < 1 << s1.k; i++) {
				int j = ((1 << s1.k) - 1) ^ i;
				max = Math.max(max, s1.max[i] + s2.max[j]);
			}
			return max;
		}

		void print() {
			for (int i = second.size() - 1; i >= 0; i--) {
				System.err.print(second.get(i) + " ");
			}
			System.err.print(" | ");
			for (State s : first) {
				System.err.print(s + " ");
			}
			System.err.println();
		}
	}

	void solve() {
		int n = in.nextInt();
		int k = in.nextInt();
		Z = in.nextInt();
		int[] p = new int[n];
		for (int i = 0; i < n; i++) {
			p[i] = in.nextInt();
		}
		int[] mask = new int[n];
		for (int i = 0; i < n; i++) {
			int cnt = in.nextInt();
			for (int j = 0; j < cnt; j++) {
				mask[i] ^= 1 << (in.nextInt() - 1);
			}
		}
		if (Z == 0) {
			for (int i = 0; i < n; i++) {
				out.println(i + 1);
			}
			return;
		}
		FIFO fifo = new FIFO(k);
		int[] ans = new int[n];
		int right = n;
		for (int left = n - 1; left >= 0; left--) {
			fifo.add(mask[left], p[left]);
			while (fifo.getMax() >= Z) {
				right--;
				fifo.remove();
			}
			ans[left] = right == n ? -1 : right + 1;
		}
		for (int answer : ans) {
			out.println(answer);
		}
	}

	void run() {
		try {
			in = new FastScanner(new File("stage.in"));
			out = new PrintWriter(new File("stage.out"));

			solve();

			out.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}

	void runIO() {

		in = new FastScanner(System.in);
		out = new PrintWriter(System.out);

		solve();

		out.close();
	}

	class FastScanner {
		BufferedReader br;
		StringTokenizer st;

		public FastScanner(File f) {
			try {
				br = new BufferedReader(new FileReader(f));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}

		public FastScanner(InputStream f) {
			br = new BufferedReader(new InputStreamReader(f));
		}

		String next() {
			while (st == null || !st.hasMoreTokens()) {
				String s = null;
				try {
					s = br.readLine();
				} catch (IOException e) {
					e.printStackTrace();
				}
				if (s == null)
					return null;
				st = new StringTokenizer(s);
			}
			return st.nextToken();
		}

		boolean hasMoreTokens() {
			while (st == null || !st.hasMoreTokens()) {
				String s = null;
				try {
					s = br.readLine();
				} catch (IOException e) {
					e.printStackTrace();
				}
				if (s == null)
					return false;
				st = new StringTokenizer(s);
			}
			return true;
		}

		int nextInt() {
			return Integer.parseInt(next());
		}

		long nextLong() {
			return Long.parseLong(next());
		}

		double nextDouble() {
			return Double.parseDouble(next());
		}
	}

	public static void main(String[] args) {
		new stage_bm().runIO();
	}
}